using FalcoEngine;
using System;
using System.Collections;
using System.Collections.Generic;

public class Bloom : ImageEffect
{
	public float luminance = 0.3f;
	public float exposure = 0.5f;
	public float blurSize = 30.0f;
	public float blurAmount = 15.0f;
	public float quality = 0.5f;

	Material material1 = null;
	Material material2 = null;
	Material material3 = null;
	Material material4 = null;
	Shader shaderBloom = null;
	Shader shaderBrightPass = null;
	Shader shaderBlurPass = null;

	RenderTexture rtBright = null;
	RenderTexture rtBlurH = null;
	RenderTexture rtBlurV = null;
	RenderTexture outRt = null;

	RenderTexture sceneRt = null;

	void Start()
	{
		quality = Mathf.Clamp(quality, 0.1f, 1.0f);

		material1 = new Material("BloomPS1");
		material2 = new Material("BloomPS2");
		material3 = new Material("BloomPS3");
		material4 = new Material("BloomPS4");

		shaderBrightPass = Shader.Load("Shaders/ImageEffects/BloomBrightPass.shader");
		shaderBlurPass = Shader.Load("Shaders/ImageEffects/GaussianBlur.shader");
		shaderBloom = Shader.Load("Shaders/ImageEffects/Bloom.shader");

		material1.shader = shaderBrightPass;
		material2.shader = shaderBlurPass;
		material3.shader = shaderBlurPass;
		material4.shader = shaderBloom;

		rtBright = new RenderTexture((int)(Screen.width * quality), (int)(Screen.height * quality), RenderTexture.TextureType.ColorOnly);
		rtBlurH = new RenderTexture((int)(Screen.width * quality), (int)(Screen.height * quality), RenderTexture.TextureType.ColorOnly);
		rtBlurV = new RenderTexture((int)(Screen.width * quality), (int)(Screen.height * quality), RenderTexture.TextureType.ColorOnly);
		outRt = new RenderTexture(Screen.width, Screen.height, RenderTexture.TextureType.ColorOnly);

		material1.SetParameter("brightPassThreshold", luminance);

		material2.SetParameter("horizontalPass", 1);
		material2.SetParameter("blurSize", blurSize);
		material2.SetParameter("sigma", blurAmount);
		material2.SetParameter("image", rtBright.colorTexture);

		material3.SetParameter("horizontalPass", 0);
		material3.SetParameter("blurSize", blurSize);
		material3.SetParameter("sigma", blurAmount);
		material3.SetParameter("image", rtBlurH.colorTexture);

		material4.SetParameter("exposure", exposure);
		material4.SetParameter("image2", rtBlurV.colorTexture);
	}
	
	public override void UpdateRT(int width, int height)
    {
		if (rtBright != null)
			rtBright.Reset((int)(width * quality), (int)(height * quality));

		if (rtBlurH != null)
			rtBlurH.Reset((int)(width * quality), (int)(height * quality));

		if (rtBlurV != null)
			rtBlurV.Reset((int)(width * quality), (int)(height * quality));

		if (outRt != null)
			outRt.Reset(width, height);
	}
	
	void OnRenderImage(RenderTexture scene, RenderTexture source)
    {
		if (sceneRt != source)
		{
			material1.SetParameter("image", source.colorTexture);
			material1.SetParameter("depth", source.depthTexture);
			material4.SetParameter("image1", source.colorTexture);
			sceneRt = source;
		}

		Graphics.Blit(material1, rtBright);
		Graphics.Blit(material2, rtBlurH);
		Graphics.Blit(material3, rtBlurV);
		Graphics.Blit(material4, outRt, true);
	}

	void OnDestroy()
    {
		if (rtBright != null)
			rtBright.Dispose();

		if (rtBlurH != null)
			rtBlurH.Dispose();

		if (rtBlurV != null)
			rtBlurV.Dispose();

		if (outRt != null)
			outRt.Dispose();
	}
}
