using FalcoEngine;
using System;
using System.Collections;
using System.Collections.Generic;

public class SSAO : ImageEffect
{
	public float strength = 50.0f;
	public float radius = 0.5f;
	public float blurSize = 10.0f;
	public float blurAmount = 5.0f;
	public float quality = 0.75f;

	Material material1 = null;
	Material material2 = null;
	Material material3 = null;
	Material material4 = null;
	Shader shaderSSAO = null;
	Shader shaderBlurPass = null;
	Shader shaderCombine = null;

	RenderTexture outRt = null;
	RenderTexture ssaoRt = null;
	RenderTexture rtBlurH = null;
	RenderTexture rtBlurV = null;
	RenderTexture sceneRt = null;

	void Start()
	{
		quality = Mathf.Clamp(quality, 0.1f, 1.0f);

		material1 = new Material("SSAO_PS1");
		material2 = new Material("SSAO_BlurH1");
		material3 = new Material("SSAO_BlurV1");
		material4 = new Material("SSAO_Combine_PS1");

		shaderSSAO = Shader.Load("Shaders/ImageEffects/SSAO.shader");
		shaderBlurPass = Shader.Load("Shaders/ImageEffects/GaussianBlur.shader");
		shaderCombine = Shader.Load("Shaders/ImageEffects/SSAOCombine.shader");

		material1.shader = shaderSSAO;
		material2.shader = shaderBlurPass;
		material3.shader = shaderBlurPass;
		material4.shader = shaderCombine;

		material1.SetParameter("strength", strength);
		material1.SetParameter("radius", radius);

		outRt = new RenderTexture(Screen.width, Screen.height, RenderTexture.TextureType.ColorOnly);
		rtBlurH = new RenderTexture((int)(Screen.width * quality), (int)(Screen.height * quality), RenderTexture.TextureType.ColorOnly);
		rtBlurV = new RenderTexture((int)(Screen.width * quality), (int)(Screen.height * quality), RenderTexture.TextureType.ColorOnly);
		ssaoRt = new RenderTexture((int)(Screen.width * quality), (int)(Screen.height * quality), RenderTexture.TextureType.ColorOnly);

		material2.SetParameter("horizontalPass", 1);
		material2.SetParameter("blurSize", blurSize);
		material2.SetParameter("sigma", blurAmount);
		material2.SetParameter("image", ssaoRt.colorTexture);

		material3.SetParameter("horizontalPass", 0);
		material3.SetParameter("blurSize", blurSize);
		material3.SetParameter("sigma", blurAmount);
		material3.SetParameter("image", rtBlurH.colorTexture);

		material4.SetParameter("tSSAO", rtBlurV.colorTexture);
	}

	public override void UpdateRT(int width, int height)
	{
		if (outRt != null)
			outRt.Reset(width, height);

		if (rtBlurH != null)
			rtBlurH.Reset((int)(width * quality), (int)(height * quality));

		if (rtBlurV != null)
			rtBlurV.Reset((int)(width * quality), (int)(height * quality));

		if (ssaoRt != null)
			ssaoRt.Reset((int)(width * quality), (int)(height * quality));
	}

	void OnRenderImage(RenderTexture scene, RenderTexture source)
	{
		if (sceneRt != source)
		{
			material1.SetParameter("tColor", source.colorTexture);
			material1.SetParameter("tDepth", scene.depthTexture);
			material4.SetParameter("tScene", source.colorTexture);
			sceneRt = source;
		}

		Graphics.Blit(material1, ssaoRt);
		Graphics.Blit(material2, rtBlurH);
		Graphics.Blit(material3, rtBlurV);
		Graphics.Blit(material4, outRt, true);
	}

	void OnDestroy()
	{
		if (ssaoRt != null)
			ssaoRt.Dispose();

		if (rtBlurH != null)
			rtBlurH.Dispose();

		if (rtBlurV != null)
			rtBlurV.Dispose();

		if (outRt != null)
			outRt.Dispose();
	}
}
