using FalcoEngine;
using System;
using System.Collections;
using System.Collections.Generic;

public class SSGI : ImageEffect
{
	public float quality = 0.75f;
	public float indirectPower = 1.3f;

	float blurSize = 2.0f;
	float blurAmount = 9.0f;

	Material material1 = null;
	Material material2 = null;
	Material material3 = null;
	Material material4 = null;

	Shader shader1 = null;
	Shader shader2 = null;
	Shader shader3 = null;

	RenderTexture outRt = null;
	RenderTexture ssgiRt = null;
	RenderTexture rtBlurH = null;
	RenderTexture rtBlurV = null;
	RenderTexture sceneRt = null;

	void Start()
	{
		quality = Mathf.Clamp(quality, 0.5f, 1.0f);

		material1 = new Material("SSGI_PP");
		material2 = new Material("SSGI_BlurH_PP");
		material3 = new Material("SSGI_BlurV_PP");
		material4 = new Material("SSGI_Combine_PP");

		shader1 = Shader.Load("Shaders/ImageEffects/SSGI.shader");
		shader2 = Shader.Load("Shaders/ImageEffects/SSGIBlur.shader");
		shader3 = Shader.Load("Shaders/ImageEffects/SSGICombine.shader");

		material1.shader = shader1;
		material2.shader = shader2;
		material3.shader = shader2;
		material4.shader = shader3;

		outRt = new RenderTexture(Screen.width, Screen.height, RenderTexture.TextureType.ColorOnly);
		ssgiRt = new RenderTexture((int)(Screen.width * quality), (int)(Screen.height * quality), RenderTexture.TextureType.ColorOnly);
		rtBlurH = new RenderTexture((int)(Screen.width * quality), (int)(Screen.height * quality), RenderTexture.TextureType.ColorOnly);
		rtBlurV = new RenderTexture((int)(Screen.width * quality), (int)(Screen.height * quality), RenderTexture.TextureType.ColorOnly);

        material2.SetParameter("_horizontalPass", 1);
        material2.SetParameter("_blurSize", blurSize);
        material2.SetParameter("_blurAmount", blurAmount);
        material2.SetParameter("_ReflectanceTexture", ssgiRt.colorTexture);

        material3.SetParameter("_horizontalPass", 0);
        material3.SetParameter("_blurSize", blurSize);
        material3.SetParameter("_blurAmount", blurAmount);
        material3.SetParameter("_ReflectanceTexture", rtBlurH.colorTexture);

        material4.SetParameter("_SSGI", rtBlurV.colorTexture);
        material4.SetParameter("_indirectPower", indirectPower);
	}

	public override void UpdateRT(int width, int height)
	{
		if (outRt != null)
			outRt.Reset(width, height);

		if (ssgiRt != null)
			ssgiRt.Reset((int)(width * quality), (int)(height * quality));

		if (rtBlurH != null)
			rtBlurH.Reset((int)(width * quality), (int)(height * quality));

		if (rtBlurV != null)
			rtBlurV.Reset((int)(width * quality), (int)(height * quality));
	}

	void OnRenderImage(RenderTexture scene, RenderTexture source)
	{
		if (sceneRt != source)
		{
			material1.SetParameter("_MainTex", source.colorTexture);
			material2.SetParameter("_MainTex", source.colorTexture);
			material3.SetParameter("_MainTex", source.colorTexture);
			material4.SetParameter("_MainTex", source.colorTexture);
			material1.SetParameter("_CameraDepthTexture", scene.depthTexture);
			material2.SetParameter("_CameraDepthTexture", scene.depthTexture);
			material3.SetParameter("_CameraDepthTexture", scene.depthTexture);

			sceneRt = source;
		}

		Graphics.Blit(material1, ssgiRt);
		Graphics.Blit(material2, rtBlurH);
		Graphics.Blit(material3, rtBlurV);
		Graphics.Blit(material4, outRt, true);

		//Graphics.Blit(material1, outRt, true);
	}

	void OnDestroy()
	{
		if (ssgiRt != null)
			ssgiRt.Dispose();

		if (rtBlurH != null)
			rtBlurH.Dispose();

		if (rtBlurV != null)
			rtBlurV.Dispose();

		if (outRt != null)
			outRt.Dispose();
	}
}
