using FalcoEngine;
using System;
using System.Collections;
using System.Collections.Generic;

public class SSR : ImageEffect
{
	public float ssrQuality = 1.0f;
	public float blurQuality = 0.5f;
	public float blurAmount = 2.0f;
	public float blurSize = 3.0f;
	public float reflectionPower = 0.75f;

	Material material1 = null;
	Material material2 = null;
	Material material3 = null;
	Material material4 = null;

	Shader shader1 = null;
	Shader shader2 = null;
	Shader shader3 = null;

	RenderTexture outRt = null;
	RenderTexture ssrRt = null;
	RenderTexture rtBlurH = null;
	RenderTexture rtBlurV = null;
	RenderTexture sceneRt = null;

	Camera camera;

	void Start()
	{
		camera = GetComponent<Camera>();

		material1 = new Material("SSR_PP");
		shader1 = Shader.Load("Shaders/ImageEffects/SSR.shader");
		
		material2 = new Material("SSRBlurH_PP");
		material3 = new Material("SSRBlurV_PP");
		shader2 = Shader.Load("Shaders/ImageEffects/SSRBlur.shader");
		
		material4 = new Material("SSRCombine_PP");
		shader3 = Shader.Load("Shaders/ImageEffects/SSRCombine.shader");

		material1.shader = shader1;
		material2.shader = shader2;
		material3.shader = shader2;
		material4.shader = shader3;

		ssrQuality = Mathf.Clamp(ssrQuality, 0.1f, 1.0f);
		blurQuality = Mathf.Clamp(blurQuality, 0.1f, 1.0f);

		outRt = new RenderTexture(Screen.width, Screen.height, RenderTexture.TextureType.ColorOnly);
		ssrRt = new RenderTexture((int)(Screen.width * ssrQuality), (int)(Screen.height * ssrQuality), RenderTexture.TextureType.ColorOnly);
		rtBlurH = new RenderTexture((int)(Screen.width * blurQuality), (int)(Screen.height * blurQuality), RenderTexture.TextureType.ColorOnly);
		rtBlurV = new RenderTexture((int)(Screen.width * blurQuality), (int)(Screen.height * blurQuality), RenderTexture.TextureType.ColorOnly);

		material2.SetParameter("_horizontalPass", 1);
		material2.SetParameter("_blurSize", blurSize);
		material2.SetParameter("_blurAmount", blurAmount);
		material2.SetParameter("_ReflectanceTexture", ssrRt.colorTexture);

		material3.SetParameter("_horizontalPass", 0);
		material3.SetParameter("_blurSize", blurSize);
		material3.SetParameter("_blurAmount", blurAmount);
		material3.SetParameter("_ReflectanceTexture", rtBlurH.colorTexture);

		material4.SetParameter("_ReflectanceTexture", rtBlurV.colorTexture);
		material4.SetParameter("_reflectionPower", reflectionPower);
	}

	public override void UpdateRT(int width, int height)
	{
		if (outRt != null)
			outRt.Reset(width, height);

		if (ssrRt != null)
			ssrRt.Reset((int)(width * ssrQuality), (int)(height * ssrQuality));

		if (rtBlurH != null)
			rtBlurH.Reset((int)(width * blurQuality), (int)(height * blurQuality));

		if (rtBlurV != null)
			rtBlurV.Reset((int)(width * blurQuality), (int)(height * blurQuality));
	}

	void OnRenderImage(RenderTexture scene, RenderTexture source)
	{
		if (sceneRt != source)
		{
			material1.SetParameter("_MainTex", source.colorTexture);
			material1.SetParameter("_CameraDepthTexture", scene.depthTexture);
			material1.SetParameter("_CameraSurfaceInfoTexture", camera.GetGBufferTexture(Camera.GBufferTexture.MetallicRoughnessAOSpecular));

			material2.SetParameter("_MainTex", source.colorTexture);
			material2.SetParameter("_CameraDepthTexture", scene.depthTexture);

			material3.SetParameter("_MainTex", source.colorTexture);
			material3.SetParameter("_CameraDepthTexture", scene.depthTexture);

			material4.SetParameter("_MainTex", source.colorTexture);

			sceneRt = source;
		}

		Graphics.Blit(material1, ssrRt);
		Graphics.Blit(material2, rtBlurH);
		Graphics.Blit(material3, rtBlurV);
		Graphics.Blit(material4, outRt, true);
	}

	void OnDestroy()
	{
		if (ssrRt != null)
			ssrRt.Dispose();

		if (rtBlurH != null)
			rtBlurH.Dispose();

		if (rtBlurV != null)
			rtBlurV.Dispose();

		if (outRt != null)
			outRt.Dispose();
	}
}
