using FalcoEngine;
using System;
using System.Collections;
using System.Collections.Generic;

public class ScreenSpaceFog : ImageEffect
{
	public float distanceOffset = 0.8f;
	public float linearGradient = 0.5f;
	public Color fogColor = Color.white;
	public bool useAmbientColor = false;
	public float density = 0.9f;

	Material material1 = null;
	Shader shaderFog = null;

	RenderTexture sceneRt = null;

	RenderTexture outRt = null;

	void Start()
	{
		material1 = new Material("FogPS1");
		shaderFog = Shader.Load("Shaders/ImageEffects/ScreenSpaceFog.shader");

		material1.shader = shaderFog;

		material1.SetParameter("DistanceOffset", distanceOffset);
		material1.SetParameter("LinearGradient", linearGradient);
		
		if (useAmbientColor)
			material1.SetParameter("FogColor", RenderSettings.ambientColor);
		else
			material1.SetParameter("FogColor", fogColor);
		
		material1.SetParameter("Density", density);

		outRt = new RenderTexture(Screen.width, Screen.height, RenderTexture.TextureType.ColorOnly);
	}

	void Update()
    {
		if (useAmbientColor)
        {
			material1.SetParameter("FogColor", RenderSettings.ambientColor);
        }
    }

	public override void UpdateRT(int width, int height)
	{
		if (outRt != null)
			outRt.Reset(width, height);
	}

	void OnRenderImage(RenderTexture scene, RenderTexture source)
	{
		if (sceneRt != source)
		{
			material1.SetParameter("image1", source.colorTexture);
			material1.SetParameter("image2", scene.depthTexture);
			sceneRt = source;
		}

		Graphics.Blit(material1, outRt, true);
	}

	void OnDestroy()
	{
		if (outRt != null)
			outRt.Dispose();
	}
}
