using FalcoEngine;
using System;
using System.Collections;
using System.Collections.Generic;

public class SunShafts : ImageEffect
{
	public Light lightSource = null;
	public float quality = 1.0f;
	public int numSamples = 100;
	public float exposure = 0.25f;
	public float density = 1.5f;
	public float sunSize = 0.01f;
	public float raysLength = 120.0f;

	Material material1 = null;
	Material material2 = null;

	Shader shader1 = null;

	RenderTexture outRt = null;
	RenderTexture brightRt = null;
	RenderTexture sceneRt = null;

	Camera camera;

	void Start()
	{
		quality = Mathf.Min(quality, 1.0f);
		quality = Mathf.Max(quality, 0.0f);

		material1 = new Material("SUN_SHAFTS_PP_PASS1");
		material2 = new Material("SUN_SHAFTS_PP_PASS2");

		shader1 = Shader.Load("Shaders/ImageEffects/SunShafts.shader");

		material1.shader = shader1;
		material2.shader = shader1;

		outRt = new RenderTexture(Screen.width, Screen.height, RenderTexture.TextureType.ColorOnly);
		brightRt = new RenderTexture((int)(Screen.width * quality), (int)(Screen.height * quality), RenderTexture.TextureType.ColorOnly);

		material1.SetParameter("raysLength", raysLength);
		material1.SetParameter("sunSize", sunSize);

		material2.SetParameter("density", density);
		material2.SetParameter("exposure", exposure);
		material2.SetParameter("numSamples", numSamples);

		material1.SetParameter("pass", 0);
		material2.SetParameter("pass", 1);

		material2.SetParameter("brightMap", brightRt.colorTexture);

		camera = GetComponent<Camera>();
	}

	public override void UpdateRT(int width, int height)
	{
		if (outRt != null)
			outRt.Reset(width, height);

		if (brightRt != null)
			brightRt.Reset((int)(width * quality), (int)(height * quality));
	}

	void OnRenderImage(RenderTexture scene, RenderTexture source)
	{
		if (sceneRt != source)
		{
			material1.SetParameter("sceneMap", source.colorTexture);
			material1.SetParameter("depthMap", scene.depthTexture);

			material2.SetParameter("sceneMap", source.colorTexture);
			material2.SetParameter("depthMap", scene.depthTexture);

			sceneRt = source;
		}

		if (lightSource != null && camera != null)
        {
			Vector3 pos = lightSource.gameObject.transform.forward * -camera.farClipDistance * 10.0f;
			Vector3 posScreen = camera.WorldToScreenPoint(pos);

			material1.SetParameter("screenLightPos", posScreen);
			material2.SetParameter("screenLightPos", posScreen);
        }

		Graphics.Blit(material1, brightRt);
		Graphics.Blit(material2, outRt, true);
	}

	void OnDestroy()
	{
		if (outRt != null)
			outRt.Dispose();
	}
}
